/*
 * Detector.h
 *
 *  Created on: 12/giu/2010
 *      Author: fisch
 */

#ifndef FACEDETECTOR_H_
#define FACEDETECTOR_H_

#include "cv.h"
#include "highgui.h"

using namespace std;
using namespace cv;

#define PI 3.14159265

#define MIN_POINTS 50
#define MIN_POINTS_POSIT 8
#define POINT_PERC 80
#define MAX_ANGLE PI/3
#define OLD_FACE 75
#define MAX_FACE 150
#define MAX_LOST 30

#define RANSAC_SAMPLES 8
#define RANSAC_DISTANCE_THRESHOLD 30.0
#define RANSAC_ITERATIONS 20

#define EHCIMODELSCALE 1.0
#define EHCIFOCUS 602

//image structure for opengl texture
struct GLImage {
    unsigned long sizeX;
    unsigned long sizeY;
    uchar *data;
};
typedef struct GLImage GLImage;

enum { FRONTALE, PROFILE_R, PROFILE_L };

class FaceDetector
{
private:
	double startMatrix[16];
	double glPositMatrix[16];

	bool modelFound;
	bool faceFound;

	int radius;
	Point center;
	float initHeadWidth;
	float referenceCoordinateX, referenceCoordinateY, referenceCoordinateZ;

	double xc, yc, xl, yl, xr, yr, x1, y1, x2, y2;
	float roll, pitch, yaw;

	int flags, count, lost;
	double resizeROI;

	Mat gray, prev_gray;
	IplImage *pyramid, *prev_pyramid;
	vector<Point3f> modelPoints;
	vector<Point2f> points, prev_points, ransacDeleted;

	CascadeClassifier frontalCascade;
	CascadeClassifier profileCascade;
	CascadeClassifier rEyeCascade;
	CascadeClassifier lEyeCascade;
	CascadeClassifier mouthCascade;

	bool detect( Mat& img, double scale );
	bool detectFace( Mat& img, double scale, int mode );
	bool update( Mat& img, double dt );
	bool correct( Mat& img );
	void draw( Mat& img );

	void updatePoint();
	void updateRadius();
	void setReferenceCoordinate( float x, float y, float z );
	void initStartMatrix( float roll, float pitch, float yaw );

	void updateGLPositMatrix( CvMatr32f rotation_matrix, CvVect32f translation_vector );

	void insertNewPoints( Mat grayOr, Rect face, int mode );
	void create3DModel( Rect face );
	void get3dModelCoordinates( int px, int py, float& fx, float& fy, float& fz, int headWidth, int headHeight );

	bool findOrientation( Mat grayOr, Rect face, float& roll, float& pitch, float& yaw, int pose );
	Point2f findCentroid( Mat gray, Rect search );
	Point2f findPupilCenter( Mat gray, Rect eye );
	Point2f findPhiltrumCenter( Mat gray, Rect mouth );

	void resizeFaceRect( Rect src, Rect& dest , Size dimMax );
	void resizeSearch( Rect src, Rect& dest , Size dimMax, float factor );

	void getPositMatrix( CvMatr32f rotation_matrix, CvVect32f translation_vector );
	void correctPoints( vector<int> outliers );

public:
	FaceDetector();
	FaceDetector( cv::String frontalCascadeName,
				  cv::String profileCascadeName,
				  cv::String rEyeCascadeName,
				  cv::String lEyeCascadeName,
				  cv::String mouthCascadeName );
	FaceDetector( cv::String cascadeName, cv::String nestedCascadeName );

	bool drawModel();
	cv::Point getPoint();
	int getRadius();
	void getReferenceCoordinate( float& x, float& y, float& z );
	void getGLOrientation( float& roll, float& pitch, float& yaw );
	void getGlPositMatrix( double myGlPositMatrix[16] );

	bool faceDetect( unsigned char* img_pointer, double scale, double angle );
};

Rect detectBiggestHaar( Mat img, CascadeClassifier cascade, Size cascadeSize, int newWidth );

void trackOptFlow( Mat prev_gray, Mat gray, IplImage* prev_pyramid, IplImage* pyramid,
				   vector<Point2f>& points, vector<Point2f>& prev_points,
				   int& flags, int& changed, int& notStatus, float& totErr );

int ransac( vector<Point2f> imagePoints, vector<Point3f> objectPoints,
			CvMatr32f rotation_matrix, CvVect32f translation_vector,
			vector<int>& outliers );

int getMatrixUsingPosit( vector<Point2f> imagePoints, vector<Point3f> objectPoints, vector<int> selectedPoints,
						 CvMatr32f rotation_matrix, CvVect32f translation_vector );

double plot2dModel( CvMatr32f rotation_matrix, CvVect32f translation_vector,
					vector<Point3f> objectPoints,vector<Point2f> imagePoints,
					vector<int>& inliers );

double getDistanceThreshold( CvMat Ma, CvMat Mp, CvMat* Mpoints, CvMat Mlook,
							 int numPoints, vector<Point2f> imagePoints );

vector<int> getRandomSet( int sampleSize, int setSize );



#endif /* FACEDETECTOR_H_ */
